;-----------------------------LICENSE NOTICE--------------------------------------------------
;  This file is part of "OH NO, My Moustache!" an Amstrad CPC 464  game developed for the CPCRetrodev2022
;  Copyright (C) 2022 Strawberry Studio. You can find us on twitter: @Strawberryabp

;
;  This program is free software: you can redistribute it and/or modify
;  it under the terms of the GNU Lesser General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU Lesser General Public License for more details.
;
;  You should have received a copy of the GNU Lesser General Public License
;  along with this program. See the license.txt archive.
;  If not, see <http://www.gnu.org/licenses/>.
;----------------------------------------------------------------------------------------------

;;; ANIMATION MANAGER
;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.h.s"
.include "globals.h.s"
.include "entity.h.s"
.include "../sys/render.h.s"

_countdown:: .db 0x16
_max_cooldown:: .db 0x16

_death_robot:: 
    .dw _spr_robot_death_00
    .dw _spr_robot_death_01
    .dw _spr_robot_death_02
    .dw _spr_robot_death_03
    .dw _spr_robot_death_04
    .dw _spr_robot_death_05
    .dw _spr_robot_death_06
    .dw _spr_robot_death_07
    .dw _spr_robot_death_08
    .dw _spr_robot_death_09
    .dw _spr_robot_death_10
    .dw _spr_robot_death_11
    .dw _spr_robot_death_12
    .dw 0x0000

_death_robot_TP:: 
    .dw _spr_robot_TP_death_00
    .dw _spr_robot_TP_death_01
    .dw _spr_robot_TP_death_02
    .dw _spr_robot_TP_death_03
    .dw _spr_robot_TP_death_04
    .dw _spr_robot_TP_death_05
    .dw _spr_robot_TP_death_06
    .dw _spr_robot_TP_death_07
    .dw _spr_robot_TP_death_08
    .dw _spr_robot_TP_death_09
    .dw _spr_robot_TP_death_10
    .dw _spr_robot_TP_death_11
    .dw _spr_robot_TP_death_12
    .dw 0x0000
_teleport_robot::
    .dw 0x0000
    .dw _spr_robot_Teleporting_0
    .dw _spr_robot_Teleporting_0
    .dw _spr_robot_Teleporting_1
    .dw _spr_robot_Teleporting_1
    .dw _spr_robot_Teleporting_2
    .dw _spr_robot_Teleporting_2
    .dw _spr_robot_Teleporting_3
    .dw _spr_robot_Teleporting_3
    .dw 0x0000
animation_manager_update::
    ld hl, #animationmanager_change_player_sprite
    ld d, #typeInput
    call isValidForSignature

    ld hl, #animationmanager_enemy_death
    ld d, #typeEnemyDead
    call isValidForSignature
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animationmanager_change_player_sprite::
    push hl
    push af
    push de
;;; NOT MOVING
    ld a, checkers(ix)
    cp #isMoving
    jr z, pop_animation
;;; NOT SHOOTING
    cp #isBulletActive
    jr z, pop_animation
;;; CHECK COUNTDOWN
    ld a, (_countdown)
    cp #0
    jr nz, endif_countdown_zero
    ;;; IF COUNTDOWN == 0 -> CHANGE SPRITE TO OTHER
        ld a, checkers(ix)
        cp #isSpriteOne
        jr z, change_to_zero
        ;;; change to one
            add #isSpriteOne
            ld de, #_spr_player_1
            call entitymanager_set_sprite
            jr endif_change_sprite
        change_to_zero:
        ;;; change to zero
            sub #isSpriteOne
            ld de, #_spr_player_0
            call entitymanager_set_sprite
            endif_change_sprite:
        ld checkers(ix), a
        ld a, (_max_cooldown)
        ld (_countdown), a
        jr pop_animation
    endif_countdown_zero:
    ;;; IF COUNTDOWN != 0 -> DECREASE COUNTDOWN
        dec a
        ld (_countdown), a


    pop_animation:
        pop de
        pop af
        pop hl
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del enemigo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animationmanager_enemy_death::
    push hl
    push af
    push de
    
    ;;; GET NUM SPRITE DE CHECKERS
    ld a, checkers(ix)
    cp #0
    jr nz, not_init_death_sequence          ;;; if (A == 0) > Explosion no comenzada
        ;;; Inicializar death sequence
        ld hl, (_death_robot)
            ;;; IF TPENEMY > _death_robot_TP
            push af
                ld a, type(ix)
                and #typeSpecialEnemy
                cp #typeSpecialEnemy
                jr nz, endif_TP_robot
                    ld hl, (_death_robot_TP)
            endif_TP_robot:
            pop af

        ex de, hl
        call entitymanager_set_sprite
        inc a
        ld checkers(ix), a
        jr endif_enemy_death

    not_init_death_sequence:                ;;; if (A != 0 ) > Mostrar siguiente sprite
        ;;; ELSE HL = SPRITE
        ld hl, #_death_robot
            push af
                ld a, type(ix)
                and #typeSpecialEnemy
                cp #typeSpecialEnemy
                jr nz, endif_TP_robot_update_death_sprite
                    ld hl, #_death_robot_TP
            endif_TP_robot_update_death_sprite:
            pop af
        call animationmanager_get_sprite
        ld c, (hl)
        inc hl
        ld b, (hl)
        push bc
        pop hl
        ex de, hl

        push af
            xor a                           ;;; if ( sprite == null ) > set_for_dead
            sub d
            sub e
            cp #0
            jr nz, endif_hl_null_ptr
                pop af                
                xor a
                ld checkers(ix), a
                
                ld a, (_num_enemies)            ;
                dec a                           ; Decrease nº of enemies
                ld (_num_enemies), a            ;

                ld type(ix), #typeDead          ; Remove Enemy
                call sys_render_erase_bullet    ;
                jr endif_enemy_death
        endif_hl_null_ptr:
        pop af

        call entitymanager_set_sprite
        ;; CHECKERS++
        inc a
        ld checkers(ix), a
    endif_enemy_death:
        pop de
        pop af
        pop hl
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del enemigo
;;; - HL = Array de sprites
;;; DEVUELVE:
;;; - HL = Sprite actual dentro del array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animationmanager_get_sprite::
    ;ld hl, #_death_robot
    ld a, checkers(ix)
    inc a
    ld c, a
    ld b, #0x00
    add hl, bc
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del enemigo
;;; - HL = Array de sprites
;;; DEVUELVE:
;;; - HL = Sprite actual dentro del array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animationmanager_get_inverse_sprite::
    ld a, checkers(ix)
    dec a
    ld c, a
    ld b, #0x00
    add hl, bc
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del enemigo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
update_tp_start_animation::
    ld a, checkers(ix)
    cp #0
    jr nz, tp_has_started          ;;; if (A == 0) > Explosion no comenzada
        ld hl, (_teleport_robot+1)
        ex de, hl
        call entitymanager_set_sprite
        inc a
        ld checkers(ix), a
        jr endif_start_animation
    tp_has_started:
    ld hl, #_teleport_robot
    call animationmanager_get_sprite
    ld c, (hl)
    inc hl
    ld b, (hl)
    push bc
    pop hl
    ex de, hl

    push af
        xor a                           ;;; if ( sprite == null ) > ANIMATION ENDED
        sub d
        sub e
        cp #0
        jr nz, endif_tp_animation_ended
            pop af               
            ld checkers(ix), #6 
            ld a, #isTPCarril
            ld datacheckers(ix), a
            jr endif_start_animation
    endif_tp_animation_ended:
        pop af
        call entitymanager_set_sprite
        inc a
        ld checkers(ix), a
    endif_start_animation:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PARAMETROS:
;;; - IX = Entidad del enemigo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
update_tp_end_animation::
    ld a, checkers(ix)
    cp #6
    jr nz, tp_end_has_initialized          ;;; if (A == 0) > Explosion no comenzada
        ld hl, (_teleport_robot+6)
        ex de, hl
        call entitymanager_set_sprite
        dec a
        ld checkers(ix), a
        jr endif_end_animation
    tp_end_has_initialized:

    ld hl, #_teleport_robot
    call animationmanager_get_inverse_sprite
    ld c, (hl)
    inc hl
    ld b, (hl)
    push bc
    pop hl
    ex de, hl

    push af
        ;xor a                           ;;; if ( sprite == null ) > ANIMATION ENDED
        ;sub d
        ;sub e
        ld a, checkers(ix)
        cp #1

        jr nz, endif_tp_end_animation_ended
            pop af  
            ld de, #_spr_robotTP_0
            call entitymanager_set_sprite

            ld a, #isTPSpeed
            ld datacheckers(ix), a
            jr endif_end_animation
    endif_tp_end_animation_ended:
        pop af
        call entitymanager_set_sprite
        dec a
        ld checkers(ix), a


    endif_end_animation:
ret