;-----------------------------LICENSE NOTICE--------------------------------------------------
;  This file is part of "OH NO, My Moustache!" an Amstrad CPC 464  game developed for the CPCRetrodev2022
;  Copyright (C) 2022 Strawberry Studio. You can find us on twitter: @Strawberryabp

;
;  This program is free software: you can redistribute it and/or modify
;  it under the terms of the GNU Lesser General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU Lesser General Public License for more details.
;
;  You should have received a copy of the GNU Lesser General Public License
;  along with this program. See the license.txt archive.
;  If not, see <http://www.gnu.org/licenses/>.
;----------------------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "globals.h.s"
.include "entity.h.s"
.include "../sys/render.h.s"
.include "game.h.s"

_num_entities:: .db 0
_max_lifes:: .db 4
;_cambio_en_vidas:: .db 1
_num_enemies::  .db 0
_num_max_enemies:: .db 1
_num_max_enemy_life:: .db 1
_last_elem_ptr:: .dw _entity_array
_entity_array::
    .ds max_entities*entity_size+1
_entity_robot_size::
    .dw 16*28
_carril_array::
    ;.db 0, 13, 26, 39
    .db 0, 13, 26, 39
    ;.db 0, 14, 28, 42
_carril_array_inv::
    ;.db 39, 26, 13, 0
    .db 39, 26, 13, 0
    ;.db 39, 26, 13, 0
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; HL = Template de la entidad que se va a crear
;;; "Devuelve":
;;; IX = Entidad que se acaba de crear
;;; A  = Si se  ha creado la entidad o no
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_create::
    ld a, (_num_entities)
    sub a, #max_entities
    ret z

    ld de, (_last_elem_ptr) ;En DE se guarda el puntero donde va a ir lo de HL por eso como HL es lo q "pasan por parametro" quiero guardarlo al final del 
                            ;"array de entidades" al final del todo
    ld bc, #entity_size     ;Las veces que va a copiar lo q hay en donde dice HL
    ldir                    ;Copia lo q hay en HL en DE, por ejemplo hay en HL 4000: 0F 35 76 E3 A2 y lo quiere copiar BC veces(5) en DE 5000

    ld ix, (_last_elem_ptr)
    ;ld de, #_spr_nave_grande
    ;call entitymanager_set_sprite

    ld a, (_num_entities)   ;
    inc a ;INCREMENTO A     ;Incrementa la cantidad de entidades q hay
    ld (_num_entities), a   ;

    ld hl, (_last_elem_ptr)  
    ld bc, #entity_size     ; Avanza el puntero al ultimo a donde se pueda crear otra entidad
    add hl, bc
    ld (_last_elem_ptr), hl ;Lo guarda en last element ptr   
    
    ld a, #0x01
ret

entitymanager_create_enemy::
    ld a, (_num_enemies)      ;;;
    ld b, a                   ;;; if ( num_enemies >= num_max_enemies ) return
    ld a, (_num_max_enemies)  ;;; 
    cp a, b                   ;;;
    ret z                     ;;;
    ret c                     ;;;

    call entitymanager_create ;;; Crea la entity

    sub a, #0x01              ;;; Comprueba si se ha creado
    ret nz                    ;;; la entidad


    ld a, (_num_enemies)   ;
    inc a ;INCREMENTO A     ;Incrementa la cantidad de entidades q hay
    ld (_num_enemies), a   ;


    ;;; Hacer el random y & 3 (Coger ultimos 2 bits)
    call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
    and #0x03                         ;;; 2 bits (3 = 0bx00000011)
    

    ;;; Coger ese indice de carril
    ld e, a
    ld d, #0x00

    ld hl, #_carril_array 
    add hl, de

    ld carril(ix), a                ;;; Almacenar el carril

    ;;; Ponerlo como posx
    ld a, (hl)
    add #2  ;;; valor que hay que cambiar para centrar los enemigos
    ld posx(ix), a 

    ;;; Cambiar el sprite segun las vidas
    
    ; Hacer random 
    ld a, (_num_max_enemy_life)    
    cp #0x01
    jr nz, life_not_one
        ;;; If a = 1 -> lifes(ix) = a

        jr endif_lifes

    life_not_one:
    cp #0x02
    jr nz, loop_random_lifes
        ;;; If a = 2 -> and 0x01 -> (== 0 -> lifes(ix) = 1) (==1 -> lifes(ix) = 2)
        call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
        and #0x01                         ;;; Cogemos solo el ultimo bit
        inc a                             ;;; Aumenta a en 1 -> rango = [1,2]
        jr endif_lifes

    loop_random_lifes:
        ;;; If a = 3 -> random and 0x03
        call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
        and #0x03                         ;;; 2 bits (3 = 0bx00000011)
        cp #0
    jr z, loop_random_lifes ; Si es 0, hacer de nuevo
    
    endif_lifes:
    ld lifes(ix), a
    call entitymanager_update_lifes_enemy
ret

entitymanager_create_special_enemy:
    ld a, (_num_enemies)      ;;;
    ld b, a                   ;;; if ( num_enemies >= num_max_enemies ) return
    ld a, (_num_max_enemies)  ;;; 
    cp a, b                   ;;;
    ret z                     ;;;
    ret c                     ;;;


    call entitymanager_create ;;; Crea la entity

    sub a, #0x01              ;;; Comprueba si se ha creado
    ret nz                    ;;; la entidad


    ld a, (_num_enemies)   ;
    inc a ;INCREMENTO A     ;Incrementa la cantidad de entidades q hay
    ld (_num_enemies), a   ;


    ;;; Hacer el random y & 3 (Coger ultimos 2 bits)
    call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
    and #0x03                         ;;; 2 bits (3 = 0b00000011)
    

    ;;; Coger ese indice de carril
    ld e, a
    ld d, #0x00

    ld hl, #_carril_array 
    add hl, de

    ld carril(ix), a                ;;; Almacenar el carril

    ;;; Ponerlo como posx
    ld a, (hl)
    add #2  ;;; valor que hay que cambiar para centrar los enemigos
    ld posx(ix), a 

    ;;; RANDOMIZAR UN NUMERO ENTRE 90 Y 150
    loop_randomize_TP_position:
        call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
        cp #199
        jr nc, loop_randomize_TP_position
        cp #120
        jr c, loop_randomize_TP_position
        ;;; 90 < A < 150
        ld data(ix), a

    ;ld lifes(ix), #1
ret


entitymanager_destroy:: ;Me pasan IX

    ;Restar 5 a lastpointer
    ld hl, (_last_elem_ptr)
    ld bc, #-entity_size
    add hl, bc
    ld (_last_elem_ptr), hl
     

    ; Copiar lastelemet en ix
    ld hl, (_last_elem_ptr)
    push ix
    pop de
    ld bc, #entity_size
    ldir


    ; Last element = invalid
    ld hl, (_last_elem_ptr)
    ld (hl), #typeInvalid

    ; decrementar num entidades
    ld a, (_num_entities)   ;
    dec a ;DECREMENTO A     ;Decrementa la cantidad de entidades q hay
    ld (_num_entities), a   ;

    

    
ret

isValid::
    ld a, (_num_entities)    ;Cargo en A los objetos q tiene entity para poder ir saltando de entidad a entidad
    ld ix, #_entity_array   ;IX es ahora
    chkValid:
        sub a, #0           ;Le resto 0
        ret z               ;Compruebo si es 0 para returnar
        push af             ;Meto af a la pila (PORQ A SOLO NO PUEDE) porque necesito A 

        ld a, type(ix)      ;Cargo en A el tipo de la entidad
        and #typeRenderable
        sub a, #typeRenderable    ;Resto el tipo de estrella correcta

        jr nz, skipHl       ;Compruebo que da 0 
            pop af
            ld bc, #seguirChkValid
            push bc
            jp (hl)         ;Va hacia lo q haya en HL (La funcion pasada por parametro)
            jr seguirChkValid
        skipHl:
        pop af
        seguirChkValid:             
        ld bc, #entity_size ;Se carga en BC la longitudo de la entidad
        add ix, bc          ;Añado a ix la longitud de la entidad para pasar a la siguiente

        ;pop af              ;Saco AF de la pila
        dec a               ;Decremeto A
        
        
    jr chkValid


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; HL = Funcion que queramos ejecutar
;;; D  = Signature que queremos seguir
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
isValidForSignature::
    ld a, (_num_entities)    ;Cargo en A los objetos q tiene entity para poder ir saltando de entidad a entidad
    ld ix, #_entity_array   ;IX es ahora
    chkValidSignature:
        sub a, #0           ;Le resto 0
        ret z               ;Compruebo si es 0 para returnar
        push af             ;Meto af a la pila (PORQ A SOLO NO PUEDE) porque necesito A 

        ld a, type(ix)      ;Cargo en A el tipo de la entidad
        and d
        sub a, d           ;LOS TIPOS COINCIDEN CON EL SIGNATURE

        jr nz, endif_comprobar_signature       ;Compruebo que da 0 
            pop af
            ld bc, #seguirchkValidSignature
            push bc
            jp (hl)         ;Va hacia lo q haya en HL (La funcion pasada por parametro)
            jr seguirchkValidSignature
        endif_comprobar_signature:
        pop af
        seguirchkValidSignature:             
        ld bc, #entity_size ;Se carga en BC la longitudo de la entidad
        add ix, bc          ;Añado a ix la longitud de la entidad para pasar a la siguiente

        ;pop af              ;Saco AF de la pila
        dec a               ;Decremeto A
        
        
    jr chkValidSignature
    
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Entidad de la que se quiere coger el sprite
;;; Devuelve:
;;; HL = Sprite de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_get_sprite::
    push de

    ld e, sprite(ix)
    ld d, sprite+1(ix)
    ex de, hl

    pop de
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Entidad de la que se quiere cambiar el sprite
;;; DE = Sprite al que lo quieres cambiar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_set_sprite::
    ld sprite(ix), e
    ld sprite+1(ix), d
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Jugador del que cambia la vida
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_update_lifes_player::
    push af
    push de

    ;; Fred recibe danyo
    push ix

    ld l, #1
    ld h, #14
    ld e, #55
    ld d, #4
    ld bc, #-0xFFFF
    ld a, #100
    call cpct_akp_SFXPlay_asm
    ld l, #1
    ld h, #14
    ld e, #50
    ld d, #3
    ld bc, #-0xFFFF
    ld a, #100
    call cpct_akp_SFXPlay_asm
    ld l, #1
    ld h, #14
    ld e, #45
    ld d, #2
    ld bc, #-0xFFFF
    ld a, #100
    call cpct_akp_SFXPlay_asm
    

    pop ix

    ld a, lifes(ix)                     ;
    cp #0                               ; if ( lifes == 0 ) return
    jr z, endif_updated_player_lifes    ;

    dec a                               ; lifes--
    ld lifes(ix), a                     ;

    jr nz, endif_updated_player_lifes   ; if ( lifes > 0 ) return

    xor a
    ld (_playerAlive), a
    

    ld de, #_spr_player_6
    call entitymanager_set_sprite

    push ix

    ld hl, #killEntity
    call isValid

    ld hl, #entitymanager_set_not_movable
    ld d, #typeMovable
    call isValidForSignature
    pop ix
    
    ;ld a, (_num_enemies)
    ;    add a
    ;ld (_num_enemies), a

    jr endif_other_sprite_player
    endif_updated_player_lifes:

            ;;; SET SPRITE TO HIT
            ld de, #_spr_player_2
            call entitymanager_set_sprite
    endif_other_sprite_player:

    call draw_lifes_player

    pop de
    pop af
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Enemigo que va a cambiar de sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_update_lifes_enemy::
    push af
    push hl
    push bc
    push de

        ld a, lifes(ix)
        ld hl, #_spr_robot0_0
        ld bc, (_entity_robot_size)
    loop_update_lifes_enemy:               ;/
        cp a, #0                           ;| while ( a > 0 )
        jr z, endwhile_update_lifes_enemy  ;\

        ;;; Add sprite index 8x28 to go to next sprite
        add hl, bc


        dec a                           ; a --
        jr loop_update_lifes_enemy      ; return to while condition
    endwhile_update_lifes_enemy:

    ;;; Set sprite of enemy as the sprite index
    ex de, hl
    call entitymanager_set_sprite

    pop de
    pop bc
    pop hl
    pop af
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Entidad del que se va a quitar el movimiento
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entitymanager_set_not_movable::
    push af

    ld a, type(ix)
    sub #typeMovable
    ld type(ix), a

    pop af
ret

entitymanager_update_enemy_spawn::

    ; IF MAX == 3 & LIFES == 3 RET
    ld a, (_num_max_enemies)                    ;
    cp a, #3                                    ; Comprobamos que el nivel de
    jr nz, endif_maximum_enemies                ; enemigos no sea el máximo
        ld a, (_num_max_enemy_life)             ;
        cp a, #3                                ;
        ret z                                   ;
    endif_maximum_enemies:
    ; IF LIFES == 1 | 2 -> LIFES++
    ld a, (_num_max_enemy_life)                 ;
    cp a, #3                                    ; if ( _num_max_enemy_life != 3 )
    jr z, endif_max_cant_vidas                  ;
        ; LIFES == 1 || LIFES == 2
        inc a                                   ; _num_max_enemy_life++
        ld (_num_max_enemy_life), a             ;

        jr endif_update_enemy_spawn
    endif_max_cant_vidas:                       ; if ( _num_max_enemy_life == 3 )
        ; LIFES == 3
        ld a, #1                                ;
        ld (_num_max_enemy_life), a             ; _num_max_enemy_life == 1

        ld a, (_num_max_enemies)                ;
        inc a                                   ; _num_max_enemies++
        ld (_num_max_enemies), a                ;
    
    endif_update_enemy_spawn:

ret



killEntity:
    ld type(ix), #typeDead
ret