;-----------------------------LICENSE NOTICE--------------------------------------------------
;  This file is part of "OH NO, My Moustache!" an Amstrad CPC 464  game developed for the CPCRetrodev2022
;  Copyright (C) 2022 Strawberry Studio. You can find us on twitter: @Strawberryabp

;
;  This program is free software: you can redistribute it and/or modify
;  it under the terms of the GNU Lesser General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU Lesser General Public License for more details.
;
;  You should have received a copy of the GNU Lesser General Public License
;  along with this program. See the license.txt archive.
;  If not, see <http://www.gnu.org/licenses/>.
;----------------------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "globals.h.s"
.include "entity.h.s"
.include "animation.h.s"
.include "game.h.s"
.include "../sys/render.h.s"
.include "../sys/physics.h.s"
.include "../sys/ai.h.s"

;;; TEMPLATES PARA LAS DISTINTAS ENTIDADES          

_playerAlive:: .db 0
_game_points:: .db 0
_game_score:: .db 48,48,48,48,48,48,0

enemigo_templ: .db  typeMovable | typeEnemy| typeRenderable   ;;; TYPE
                   .db   2                                        ;;; LIFES
                   .db   0,172                                    ;;; X, Y
                   .db   1                                        ;;; CARRIL
                   .db   8, 28                                    ;;; W/2, H
                   .db   0, -1                                    ;;; VX, VY
                   .db   0                                        ;;; CHECKERS
                   .dw  _spr_robot0_0                             ;;; SPRITE
                   .dw  0x0000                                    ;;; DATA
enemigo_especial_templ: .db typeMovable | typeSpecialEnemy| typeEnemy | typeRenderable   ;;; TYPE
                   .db   1                                        ;;; LIFES
                   .db   0,172                                    ;;; X, Y
                   .db   1                                        ;;; CARRIL
                   .db   8, 30                                    ;;; W/2, H
                   .db   0, -1                                    ;;; VX, VY
                   .db   0                                        ;;; CHECKERS
                   .dw  _spr_robotTP_0                           ;;; SPRITE
                   .dw  0x0000                                    ;;; DATA

player_templ:      .db  typeInput | typeMovable | typeRenderable  ;;; TYPE
                   .db   4                                        ;;; LIFES
                   .db   0,   6                                   ;;; X, Y
                   .db   0                                        ;;; CARRIL
                   .db  12,  44                                   ;;; W/2, H
                   .db   0,   0                                   ;;; VX, VY
                   .db   0                                        ;;; CHECKERS
                   .dw  _spr_player_0                             ;;; SPRITE
                   .dw  0x0000                                    ;;; DATA

wait:
   ld b, #2
   ld a, #1
   loopwait:
      push af
      call cpct_waitHalts_asm
      call cpct_waitVSYNC_asm
      call cpct_akp_musicPlay_asm
      pop af
      dec a
      ret z
   jr loopwait


gamemanager_init::
   ld a, #1
   ld (_playerAlive), a

   call init_render

   ld de, #_song_game
   call cpct_akp_musicInit_asm

   ld de, #_song_game
   call cpct_akp_SFXInit_asm
ret

gamemanager_play::

   ld hl, #player_templ             ;;; Create player spaceship
   call entitymanager_create        ;;;
   call draw_lifes_player
   loop:
      ld ix, #_entity_array

   call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
   and #0x15
   cp #0
   jr z, createSpecialEnemy
   ld hl, #enemigo_templ        ;;; Create enemy spaceship

   call entitymanager_create_enemy  ;;;
   jr noSpecialEnemies

   createSpecialEnemy:
      ld hl, #enemigo_especial_templ
      call entitymanager_create_special_enemy
   noSpecialEnemies:
      call sys_ai_update            ;;; Update VX for AI entities
      call updateAllEntities        ;;; Update POSX for MOVABLE entities

      ld hl, #updateSpecialEnemies
      ld d, #typeSpecialEnemy
      call isValidForSignature
      

      call wait
      call update_render            ;;; Render al RENDERABLE entities

      ;ld hl, #nave_grande_templ        ;;; Create enemy spaceship
      ;call entitymanager_create_enemy
      call animation_manager_update

      call chkDead                  ;;; Checks if it needs to DESTROY an entity
      ld a, (_playerAlive)
      cp #0
      jr nz, loop

      call destroy_points


ret


destroy_points::
   xor a
   ld (_game_points), a
   ld (_num_enemies), a

   ld a, #6
   ld hl, #_game_score
   loop_reset_points:
      ld (hl),#48
      inc hl
      dec a
   jr nz, loop_reset_points

   ld a, #1
   ld (_num_max_enemies),a
   ld (_num_max_enemy_life), a 

   ld a, (_max_cooldown)
   ld (_countdown), a

ret