;-----------------------------LICENSE NOTICE--------------------------------------------------
;  This file is part of "OH NO, My Moustache!" an Amstrad CPC 464  game developed for the CPCRetrodev2022
;  Copyright (C) 2022 Strawberry Studio. You can find us on twitter: @Strawberryabp

;
;  This program is free software: you can redistribute it and/or modify
;  it under the terms of the GNU Lesser General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU Lesser General Public License for more details.
;
;  You should have received a copy of the GNU Lesser General Public License
;  along with this program. See the license.txt archive.
;  If not, see <http://www.gnu.org/licenses/>.
;----------------------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "../man/entity.h.s"
.include "../man/animation.h.s"
.include "../man/game.h.s"
.include "../globals.h.s"
.include "render.h.s"


bullet_templ:      .db  typeBullet | typeMovable | typeRenderable ;;; TYPE
                   .db   1                                        ;;; ATTACK
                   .db   0,  56                                   ;;; X, Y 
                   .db   1                                        ;;; CARRIL
                   .db   4,  12                                   ;;; W/2, H
                   .db   0,   2                                   ;;; VX, VY
                   .db   0                                        ;;; CHECKERS
                   .dw  _spr_bullet                               ;;; SPRITE
                   .dw  0x0000                                    ;;; DATA


updateOneEntity::
    push af
    push hl

    ;;; COMPROBAR SI ES DE TIPO INPUT
    ld a, type(ix)
    and #typeInput
    cp #typeInput
    jr nz, endif_type_input
        ;;; SI ES TIPO INPUT
        ;call sys_physics_check_keyboard
        call chkMove
    endif_type_input: 
    ld a, posx(ix)  ;
    add a, vx(ix)   ;   Cambia la pos segun la velocidad
    ld posx(ix), a  ;

    ld a, posy(ix)  ;
    add a, vy(ix)   ;   Cambia la pos segun la velocidad
    ld posy(ix), a  ;

    pop hl
    pop af
ret

chkDead::
    ld ix, #_entity_array
    dead:
        ld a, type(ix)      ;
        sub a, #typeInvalid ;Comprobar que no sea la ultima
        ret z               ;

        ld a, type(ix)
        sub a, #typeDead
        jr nz, nextEntity
            call entitymanager_destroy
        jr dead
        nextEntity:
            ld bc, #entity_size ;Se carga en BC la longitudo de la entidad
            add ix, bc          ;Añado a ix la longitud de la entidad para pasar a la siguiente
    jr dead

updateAllEntities::
    ld hl, #updateOneEntity
    ld d, #typeMovable
    call isValidForSignature

    ld hl, #sys_physics_check_collision
    ld d, #typeBullet | #typeMovable
    call isValidForSignature

    ld hl, #chkEnemyReach
    ld d, #typeEnemy
    call isValidForSignature
ret

updateSpecialEnemies::
    push af
    push de
    push hl
    ld a, vy(ix)
    cp #0
    jr nz, addSpeed
    dec a
    jr skipAddingSpeed

    addSpeed:


        inc a
    skipAddingSpeed:
    ld vy(ix), a

    ld a, posy(ix)
    cp #90
    jr c, noTP   
        ;call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
        ;cp #0
        ;jr nz, noTP
        cp data(ix)
        jr nz, noTP
            ;;; VELOCIDAD = 0 
            ld vy(ix), #0x00
            ;;; SI TODO == 0 > ENCENDER ANIMACION TP
            ld a, datacheckers(ix)
            cp #0
            jr nz, animation_has_started
                ld a, #isTPStart
                ld datacheckers(ix),a 
                jr noTP
            animation_has_started:
            ;;; SI ANIMACION TP == 1 > UPDATE ANIMACION TP
            and #isTPStart
            cp #isTPStart
            jr nz, start_animation_has_ended
                call update_tp_start_animation
                jr noTP
            start_animation_has_ended:
            ;;; SI CAMBIAR CARRIL == 1 > CAMBIAR CARRIL
            ld a, datacheckers(ix)
            and #isTPCarril
            cp  #isTPCarril
            jr nz, has_changed_carril
                loop_change_carril_TP_enemy:
                call cpct_getRandom_xsp40_u8_asm  ;;; Mete en A un random que ocupa 
                and #0x03
                ld b, carril(ix)
                cp b
                jr z, loop_change_carril_TP_enemy
                    ;;; SPEED = 0
                    ;;; ACTIVATE TP CHECKERS
                    call sys_render_erase_bullet
                    ld carril(ix), a
                    ld hl, #_carril_array
                    ld b, #0x00
                    ld c, carril(ix)
                    add hl, bc
                    ld a, (hl)
                    add a, #2
                    ld posx(ix), a

                    ld a, #isTPEnd
                    ld datacheckers(ix), a
                    jr noTP
            has_changed_carril:
            ;;; SI ANIMACION TP 2 == 1 > UPDATE ANIMACION TP 2
            ld a, datacheckers(ix)
            and #isTPEnd
            cp #isTPEnd
            jr nz, end_animation_has_ended
                call update_tp_end_animation
                jr noTP
            end_animation_has_ended:
            ;;; SI VELOCIDAD REACTIVA == 1 > VELOCIDAD = 1
            ld a, datacheckers(ix)
            and #isTPSpeed
            cp #isTPSpeed
            jr nz, end_speed_update
                call sys_render_erase_bullet
                ld vy(ix), #-1
                ld a, #isTPDone
                ld datacheckers(ix), a
            end_speed_update:

    noTP:
    pop hl
    pop de
    pop af
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Entidad qde la que se va a mirar el input
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_physics_check_keyboard::
    push de
    push af

    call cpct_scanKeyboard_f_asm
    ld vx(ix), #0

    ld hl, #Joy0_Left
    call cpct_isKeyPressed_asm
    jr nz, left_key_pressed
    
    ld hl, #Key_O
    call cpct_isKeyPressed_asm
    jr z, endif_keyO_pressed
        ;;; KEY O WAS PRESSED
        ;call borrarPlayer
        left_key_pressed:
        ld a, posx(ix)
        sub #0
        jr z, endif_keyO_pressed
        ld vx(ix), #-1
            ;;; SET SPRITE TO LEFT
            ld de, #_spr_player_3
            call entitymanager_set_sprite
        ld a, checkers(ix)
        or #isMoving
        ld checkers(ix), a
        
    endif_keyO_pressed:


    ld hl, #Joy0_Right
    call cpct_isKeyPressed_asm
    jr nz, right_key_pressed

    ld hl, #Key_P
    call cpct_isKeyPressed_asm
    jr z, endif_keyP_pressed
        ;;; KEY P WAS PRESSED
        ;call borrarPlayer
        right_key_pressed:
        ld a, posx(ix)
        sub #39
        ;sub #42
        jr z, endif_keyP_pressed
        ld vx(ix), #1
            ;;; SET SPRITE TO RIGHT
            ld de, #_spr_player_4
            call entitymanager_set_sprite
        ld a, checkers(ix)
        or #isMoving
        ld checkers(ix), a
    endif_keyP_pressed:  
    
    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    jr nz, fire_has_been_pressed  
    ld hl, #Joy0_Fire2
    call cpct_isKeyPressed_asm
    jr nz, fire_has_been_pressed  

    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    jr z, if_keySpace_NOT_pressed

        fire_has_been_pressed:
        ;;; CHECKEAR SI HAY BALA ACTIVA

        ld a, checkers(ix)                  ;Cargo en A el tipo de la entidad
        and #isBulletActive
        sub a, #isBulletActive           ;LOS TIPOS COINCIDEN CON EL SIGNATURE
        jr z, endif_keySpace_pressed

        ;;; KEY SPACE WAS PRESSED
        push ix
        pop iy

        ld a, checkers(ix)
        and #0b11111011
        ld checkers(ix), a
        ld hl, #chkCanIShoot
        ld d, #typeBullet
        call isValidForSignature
        push iy
        pop ix
        ld a, checkers(ix)
        and #isShootCooldownActive
        sub a, #isShootCooldownActive
        jr z, endif_keySpace_pressed

        push iy
        ld a, posx(iy)
        ld b, carril(iy)
        push bc
        ld hl, #bullet_templ
        push af
        call entitymanager_create
        pop af
        pop bc
        add a, #4
        ld carril(ix), b
        ld posx(ix), a
        pop ix

        ;;Sprite lanzar

            ;;; SET SPRITE TO THROW
            ld de, #_spr_player_5
            call entitymanager_set_sprite

        ;;Chupito lanzado
        push ix
        ld l, #2
        ld h, #13
        ld e, #70
        ld d, #1
        ld bc, #-0xFFF
        ld a, #100
        call cpct_akp_SFXPlay_asm
        pop ix

        ;;; BALA ACTIVADA
        ld a, checkers(ix)
        add #isBulletActive
        ld checkers(ix), a

        ;;;
        jr endif_keySpace_pressed
    if_keySpace_NOT_pressed:
        ;;; SI BALA ACTIVA

        ld a, checkers(ix)                  ;Cargo en A el tipo de la entidad
        and #isBulletActive
        sub a, #isBulletActive           ;LOS TIPOS COINCIDEN CON EL SIGNATURE
        jr nz, endif_keySpace_pressed

        ;;; DESACTIVAR BALA
        ld a, checkers(ix)
        sub #isBulletActive
        ld checkers(ix), a

            cp #isMoving
            jr z, endif_keySpace_pressed
            ;;; SET SPRITE TO IDLE
            ;ld de, #_spr_player_0
            ;call entitymanager_set_sprite

    endif_keySpace_pressed:
    pop af
    pop de
ret

sys_physics_check_collision::
    push ix
    push hl
    push af
    push de

    call chkBulletReach ;;; COMPRUEBA BALA CON FIN DE PANTALLA


    ;;; COMPROBAR COLISION DE LA BALA CON UN ENEMIGO

    push ix                 ;IX de bala
    pop iy                  ;GUARDO en IY la BALA

    ld a, (_num_entities)   ;Cargo en A los objetos q tiene entity para poder ir saltando de entidad a entidad
    ld ix, #_entity_array   ;IX DE ENTIDAD
    chkEnemyColision:
        sub #0                  ;Le resto 0
        jr z, finCheckColision  ;Compruebo si es 0 para returnar
        push af                 ;Meto af a la pila (PORQ A SOLO NO PUEDE) porque necesito A 

        ld a, type(ix)          ;Cargo en A el tipo de la ENTIDAD
        and #typeEnemy
        sub a, #typeEnemy       ;LOS TIPOS COINCIDEN CON EL SIGNATURE
        jr nz, skipCheckings    ;Compruebo que da 0 
            ; LA ENTIDAD ES UN ENEMIGO
            ld b, carril(ix)    ;carril de la ENTIDAD
            ld c, posy(ix)      ;posicion Y de la ENTIDAD

            ld a, b
            sub a, carril(iy)                       ;Carril de la BALA
            jr nz, noColition                       ; if (carril(ENTIDAD) == carril(BALA)) 
                ; CARRILES SON IGUALES
                ld a, c
                sub posy(iy)                        ;Pos Y de la BALA
                jr nc, noColition                   ; if (posy(ENTIDAD) == posy(BALA))
                    ; POSY SON IGUALES 
                    ;ld type(ix), #typeDead          ;Tipo de la ENTIDAD
                    ;call sys_render_erase_bullet

                    ;;Enemigo golpeado
                    push ix

                    ld l, #1
                    ld h, #12
                    ld e, #50
                    ld d, #0
                    ld bc, #0xFFFF
                    ld a, #100
                    call cpct_akp_SFXPlay_asm

                    pop ix

                    call chkEnemyLifes
                    push ix

                    push iy                         ; Metemos la BALA en IX
                    pop ix                          ; IX = IY
                    ld type(ix), #typeDead          ;Tipo de la BALA
        call sys_render_erase_bullet

                    pop ix
                    ;push ix                         ;Guardo el IX de la BALA
                    ;push hl     ;/ ld ix, hl
                    ;pop ix      ;\

                    pop af
                    jr finCheckColision

            noColition:
                ;push ix                             ;Guardo IX de la BALA
        skipCheckings:
        pop af   
        ld bc, #entity_size ;Se carga en BC la longitudo de la entidad
        add ix, bc          ;Añado a ix la longitud de la entidad para pasar a la siguiente

        dec a               ;Decremeto A
        
        
    jr chkEnemyColision

    finCheckColision:
    ;pop ix

    pop de
    pop af
    pop hl
    pop ix
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parametros:
;;; IX = Enemigo del que checkeamos la vida
;;; IY = Bala con la que ha colisionado
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
chkEnemyLifes::
    push af

    ld a, lifes(ix)            ;
    cp attack(iy)              ; lifes(enemy) -= attack(bullet)

    jr c, enemy_is_dead
    jr z, enemy_is_dead
        ;;; ENEMY IS ALIVE
        sub attack(iy)              ;/ lifes(enemy) -= attack(bullet)
        ld lifes(ix), a             ;\
        call entitymanager_update_lifes_enemy   ;Change Enemy sprite

        jr endif_enemy_life
    enemy_is_dead:
        ld a, type(ix)
        and #typeSpecialEnemy
        cp #typeSpecialEnemy
        jr z, add_TP_robot_to_dead_animation
            ;;; NOT A SPECIAL ENEMY
            ld a, #typeEnemyDead
            or #typeRenderable
            jr endif_TP_robot_death_animation
        add_TP_robot_to_dead_animation:
            ;;; IS A SPECIAL ENEMY
            ld checkers(ix), #0x00
            ld a, #isTPDone
            ld datacheckers(ix),a 
            ld a, #typeEnemyDead
            or #typeRenderable
            or #typeSpecialEnemy
        endif_TP_robot_death_animation:

        ld type(ix), a      ; Remove Enemy

        ;call sys_render_erase_bullet    ;
        ld vy(ix), #0


        call update_game_points         ; Increases points

        call chkUpdateDificulty         ; May increase dificulty
    endif_enemy_life:

    pop af
ret

borrarPlayer:
    ld de, #0xC000
    ld c, posx(ix)
    ld b, posy(ix)
    call cpct_getScreenPtr_asm

    ex de, hl
    ld a, #00
    ld c, width(ix)
    ld b, height(ix)
    call cpct_drawSolidBox_asm

ret


chkMove:
    ;;; check if moves
    
    ld a, checkers(ix);
    and #isMoving
    dec a
    jr z, skipToCheckWhereToMove
        call sys_physics_check_keyboard
        ret
    skipToCheckWhereToMove:
    ld a, #carril_size
    ld b, posx(ix)
    push ix
    ld ix, #_carril_array_inv
    bucle:
        push af
        ld a, (ix)
        sub b
        jr nz, sigueMoviendote
            pop af
            pop ix
            jr llegue
        sigueMoviendote:
        inc ix
        pop af
        add #-1
    jr c, bucle
    pop ix
    ret


    llegue:
        ld carril(ix), a
        ld checkers(ix), #2
        ld vx(ix), #0
            ;;; SET SPRITE TO IDLE
            ;ld de, #_spr_player_0
            ;call entitymanager_set_sprite
ret

chkBulletReach:
    ;;; COMPROBAR COLISION DE LA BALA CON EL FINAL DE LA PANTALLA
    ld a, #199                          ;;; / Posicion maxima para el sprite en pantalla
    sub height(ix)                      ;;; \

    sub posy(ix)                        ;;; / if(posicion > posicion_maxima)
    ret nc                              ;;; \

        ;ld a, #typeDead                ;;; Ponerlo como muerto
        ld type(ix), #typeDead   
        call sys_render_erase_bullet
        
ret

chkEnemyReach:
    push de
    push hl
    push af

    ;;; COMPROBAR COLISION DEL ENEMIGO CON LA BARRA
    ld a, #55                          ;;; / Posicion min para el sprite en pantalla

    sub posy(ix)                        ;;; / if(posicion > posicion_maxima)
    jr c, endif_colisiona_start_pantalla ;;; \

        ;ld a, #typeDead                   ;;; Ponerlo como muerto
        ld type(ix), #typeDead   
        
        ld a, (_num_enemies)
        dec a
        ld (_num_enemies), a

        call sys_render_erase_bullet

        ld hl, #entitymanager_update_lifes_player
        ld d, #typeInput | #typeMovable
        call isValidForSignature

    endif_colisiona_start_pantalla:
    pop af
    pop hl
    pop de
ret

chkCanIShoot:
    push af
    ld a, posy(ix) ;Cargo en A la posicion de la BALA
    ;Comprueba que la bala no haya pasado de la mitad de la pantalla
    sub a, #75    
    jr nc, doPop 
    ;Compruebo que no este ya cambiado 1 el cooldown de disparo (No podria disparar)
    ld a, checkers(iy)
    and #isShootCooldownActive
    cp a, #isShootCooldownActive
    jr z, doPop
    ;Activa a 1 el disparo porque no lo estaba antes
    ld a, checkers(iy)
    add a, #isShootCooldownActive
    ld checkers(iy), a
    doPop:
    pop af

ret

chkUpdateDificulty::
    ld a, (_game_points)
    cp a, #5
    jr nz, skip_5
        call entitymanager_update_enemy_spawn   ; MAX = 1, LIFES = 2
        ret
    skip_5:
    cp a, #10
    jr nz, skip_10
        call entitymanager_update_enemy_spawn   ; MAX = 1, LIFES = 3
        ret
    skip_10:
    cp a, #20
    jr nz, skip_20
        call entitymanager_update_enemy_spawn   ; MAX = 2, LIFES = 1
        ret
    skip_20:
    cp a, #35
    jr nz, skip_35
        call entitymanager_update_enemy_spawn   ; MAX = 2, LIFES = 2
        ret
    skip_35:
    cp a, #50
    jr nz, skip_50
        call entitymanager_update_enemy_spawn   ; MAX = 2, LIFES = 3
        ret
    skip_50:
    cp a, #75
    jr nz, skip_75
        call entitymanager_update_enemy_spawn   ; MAX = 3, LIFES = 1
        ret
    skip_75:
    cp a, #130
    jr nz, skip_130
        call entitymanager_update_enemy_spawn   ; MAX = 3, LIFES = 2
        ret
    skip_130:
    cp a, #160
    jr nz, skip_160
        call entitymanager_update_enemy_spawn   ; MAX = 3, LIFES = 3
        ret
    skip_160:

ret

update_game_points::
    ld a, (_game_points)
    inc a
    ld (_game_points), a
    ld bc, #3
    loop_update_game_score:    
        ld hl, #_game_score         ;
        add hl, bc                  ; HL = _game_score + BC

        ld a, (hl)                  ;
        cp a, #57                   ; if (HL == 9)
        jr nz, if_number_not_nine   ;
            ;;; HL == 9 
            ld (hl),#48             ; HL = 0
            dec bc                  ; BC-- (ir un digito a la izda)
            jr loop_update_game_score
        if_number_not_nine:
            ;;; HL != 9
            inc (hl)                ; HL++
    
    push iy
    call draw_points_player
    pop iy
ret