;-----------------------------LICENSE NOTICE--------------------------------------------------
;  This file is part of "OH NO, My Moustache!" an Amstrad CPC 464  game developed for the CPCRetrodev2022
;  Copyright (C) 2022 Strawberry Studio. You can find us on twitter: @Strawberryabp

;
;  This program is free software: you can redistribute it and/or modify
;  it under the terms of the GNU Lesser General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU Lesser General Public License for more details.
;
;  You should have received a copy of the GNU Lesser General Public License
;  along with this program. See the license.txt archive.
;  If not, see <http://www.gnu.org/licenses/>.
;----------------------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "globals.h.s"
.include "physics.h.s"
.include "../man/entity.h.s"
.include "../man/game.h.s"
;.include "../sprites/escenario_z.h.s"

_puntuacion_string:: .asciz "Score"
_vidas_string:: .asciz "Lives"
init_render::

    call dibujar_fondo
    call dibujar_menu

    call draw_lifes_player
    call draw_points_player
ret

;input ix: puntero a laprimera entidad
;      a: numero de entidades
update_render::

    ld hl, #draw_sprite_render
    ld d, #typeRenderable
    call isValidForSignature

    ;call draw_points_player
ret

dibujar_fondo::
    ;ld hl,  #_escenario_z_end
    ;ld de, #0xFFFF
    ;call cpct_zx7b_decrunch_s_asm

    ld de, #0xC000
    ld c, #0
    ld b, c
    call cpct_getScreenPtr_asm  

    ex de, hl

    ld hl, #_spr_fondo_0
    ld  c, #40
    ld  b, #200
    call cpct_drawSprite_asm

    ld de, #0xC000
    ld c, #40
    ld b, #0
    call cpct_getScreenPtr_asm  

    ex de, hl

    ld hl, #_spr_fondo_1
    ld  c, #40
    ld  b, #200
    call cpct_drawSprite_asm
ret

dibujar_menu::

    ;ld hl, #0x07                    ;  Texto = verde
    ld h, #0x07
    ld l,#0x00
    call cpct_setDrawCharM0_asm     ;

    ld de, #0xC000                  ;/
    ld c, #55                       ;|
    ld b, #131                      ;| Imprimir "Score:"
    call cpct_getScreenPtr_asm      ;| en la mitad del menu
                                    ;|
    ld iy, #_puntuacion_string      ;|
    call cpct_drawStringM0_asm      ;\

    ld de, #0xC000                  ;/
    ld c, #56                       ;|
    ld b, #52                       ;1 Imprimir "Lifes:"
    call cpct_getScreenPtr_asm      ;| al principio del
                                    ;| menu
    ld iy, #_vidas_string           ;|
    call cpct_drawStringM0_asm      ;\

ret

draw_points_player::
    ;ld hl, #0x00D                   ;  Texto = amarillo
    ld h, #0x03
    ld l, #0x0D
    call cpct_setDrawCharM0_asm     ;

    ld de, #0xC000                  ;/
    ld c, #54                       ;|
    ld b, #160                      ;| Imprimir los puntos
    call cpct_getScreenPtr_asm      ;| del jugador
                                    ;|
    ld iy, #_game_score             ;|
    call cpct_drawStringM0_asm      ;\
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parámetros:
;;; - IX = player del que cogemos la vida
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
draw_lifes_player::
    ;;; Iniciar datos
    xor a   ;a == 0
    ld c, #48                              ; c = pos_inicial_menu
    push bc

    ;;; Empezar loop
    loop_lifes_player:
        ;;; Comprobar cont == max_vidas
        push af                             ;
        ld a, (_max_lifes)                  ; d = _max_lifes
        ld d, a                             ; a = contador
        pop af                              ;

        pop bc                              ;
        push af                             ;
        ld a, c                             ;
        add a, #6                          ; c += 11
        ld c, a                             ;
        pop af                              ;
        push bc                             ;

        cp a, d
        jr nz, endif_has_no_max_lifes
            ;;; cont == max_vidas -> ret
            ;xor a
            ;ld (_cambio_en_vidas), a
            pop bc
            ret                              

        endif_has_no_max_lifes:
        ;VMEM donde queremos pintar
        push af

        ld de, #0xC000
        ld b, #68
        call cpct_getScreenPtr_asm  
        ex de, hl

        pop af
        ;;; cont != max vidas
            ;;; Comprobar cont < lifes(ix)
            cp a, lifes(ix)
            jr nc, endif_no_more_lifes
                ;;; cont < lifes(ix) -> pintar _spr_life_0
                ld hl, #_spr_life_0
                jr endif_draw_lifes_player
            
            endif_no_more_lifes:
                ;;; cont >= lifes(ix) -> pintar _spr_life_1
                ld hl, #_spr_life_1
        endif_draw_lifes_player:
        push af

        ld c, #5                            ;;; Ancho del sprite
        ld b, #44                           ;;; Alto del sprite
        call cpct_drawSprite_asm

        pop af
        inc a                               ;;; cont++

    jr loop_lifes_player
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parámetros:
;;; - IX = para coger sprite, width y height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
draw_sprite_render::
    push hl
    push af
    push de

    ld de, #0xC000
    ld c, posx(ix)
    ld b, posy(ix)
    call cpct_getScreenPtr_asm  

    ex de, hl

    call entitymanager_get_sprite
    ld  c, width(ix)
    ld  b, height(ix)
    call cpct_drawSprite_asm

    pop de
    pop af
    pop hl
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Parámetros:
;;; - IX = para coger x, y, width y height del sprite 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_erase_bullet::
    push hl
    push af

    ld de, #0xC000
    ld c, posx(ix)
    ld b, posy(ix)
    call cpct_getScreenPtr_asm  

    ex de, hl

    ld  a, #0xCC
    ld  c, width(ix)
    ld  b, height(ix)
    call cpct_drawSolidBox_asm

    pop af
    pop hl
ret